@@ -1,9 +1,10 @@
-import io
 import json
 import os
 import random
+from io import StringIO
 from typing import Dict, Iterator, List, Optional
 
+from ape.logging import logger
 from ape.utils import USER_AGENT
 from requests import Session
 
@@ -47,6 +48,12 @@ def get_etherscan_uri(ecosystem_name: str, network_name: str):
         )
     elif ecosystem_name == "avalanche":
         return "https://snowtrace.io"
+    elif ecosystem_name == "bsc":
+        return (
+            f"https://{network_name}.bscscan.com"
+            if network_name != "mainnet"
+            else "https://bscscan.com"
+        )
 
     raise UnsupportedEcosystemError(ecosystem_name)
 
@@ -86,6 +93,12 @@ def get_etherscan_api_uri(ecosystem_name: str, network_name: str):
         )
     elif ecosystem_name == "avalanche":
         return "https://api.snowtrace.io/api"
+    elif ecosystem_name == "bsc":
+        return (
+            f"https://api-{network_name}.bscscan.com/api"
+            if network_name != "mainnet"
+            else "https://api.bscscan.com/api"
+        )
 
     raise UnsupportedEcosystemError(ecosystem_name)
 
@@ -115,7 +128,10 @@ class _APIClient:
     ) -> EtherscanResponse:
         params = self.__authorize(params)
         return self._request(
-            "GET", params=params, headers=headers, raise_on_exceptions=raise_on_exceptions
+            "GET",
+            params=params,
+            headers=headers,
+            raise_on_exceptions=raise_on_exceptions,
         )
 
     def _post(
@@ -139,6 +155,8 @@ class _APIClient:
 
         if raise_on_exceptions:
             response.raise_for_status()
+        elif not 200 <= response.status_code < 300:
+            logger.error(response.text)
 
         return EtherscanResponse(response, self._ecosystem_name, raise_on_exceptions)
 
@@ -162,7 +180,11 @@ class ContractClient(_APIClient):
         super().__init__(ecosystem_name, network_name, "contract")
 
     def get_source_code(self) -> SourceCodeResponse:
-        params = {**self.base_params, "action": "getsourcecode", "address": self._address}
+        params = {
+            **self.base_params,
+            "action": "getsourcecode",
+            "address": self._address,
+        }
         result = self._get(params=params)
         result_list = result.value or []
 
@@ -211,7 +233,7 @@ class ContractClient(_APIClient):
             "licenseType": license_type,
             "optimizationUsed": int(optimization_used),
             "runs": optimization_runs,
-            "sourceCode": io.StringIO(json.dumps(standard_json_output)),
+            "sourceCode": StringIO(json.dumps(standard_json_output)),
         }
 
         iterator = 1
