@@ -1,8 +1,9 @@
 import json
 import os
+from io import StringIO
 from pathlib import Path
 from tempfile import mkdtemp
-from typing import IO, Dict, Union
+from typing import IO, Any, Callable, Dict, Optional, Union
 
 import _io  # type: ignore
 import ape
@@ -38,7 +39,7 @@ EXPECTED_ACCOUNT_TXNS_PARAMS = {
     "sort": "asc",
 }
 FOO_SOURCE_CODE = """
-// SPDX-License-Identifier: MIT
+// SPDX-License-Identifier: AGPL-3.0
 pragma solidity ^0.8.2;
 
 import "@bar/bar.sol";
@@ -46,14 +47,14 @@ import "@bar/bar.sol";
 contract foo {
 }
 """
-BAR_SOURCE_CODE = """
-// SPDX-License-Identifier: MIT
+BAR_SOURCE_CODE = r"""
+// SPDX-License-Identifier: AGPL-3.0
 pragma solidity ^0.8.2;
 
 contract bar {
 }
 """
-APE_CONFIG_FILE = """
+APE_CONFIG_FILE = r"""
 dependencies:
   - name: bar
     local: ./bar
@@ -62,11 +63,26 @@ solidity:
   import_remapping:
     - "@bar=bar"
 """
+STANDARD_INPUT_JSON = {
+    "language": "Solidity",
+    "sources": {
+        "foo.sol": {"content": FOO_SOURCE_CODE},
+        ".cache/bar/local/bar.sol": {"content": BAR_SOURCE_CODE},
+    },
+    "settings": {
+        "optimizer": {"enabled": True, "runs": 200},
+        "outputSelection": {
+            "foo.sol": {"foo": ["abi", "bin", "bin-runtime", "devdoc", "userdoc"]},
+            "bar.sol": {"bar": ["abi", "bin", "bin-runtime", "devdoc", "userdoc"]},
+        },
+        "remappings": ["@bar=.cache/bar/local"],
+    },
+}
 
 
 @pytest.fixture(autouse=True)
 def connection():
-    with ape.networks.parse_network_choice("ethereum:mainnet:infura") as provider:
+    with ape.networks.ethereum.mainnet.use_provider("infura") as provider:
         yield provider
 
 
@@ -76,12 +92,11 @@ def make_source(base_dir: Path, name: str, content: str):
     source_file.write_text(content)
 
 
-@pytest.fixture(autouse=True)
-def with_source_file():
+@pytest.fixture(scope="session", autouse=True)
+def project():
     base_dir = ape.config.PROJECT_FOLDER
     contracts_dir = base_dir / "contracts"
     dependency_contracts_dir = base_dir / "bar" / "contracts"
-    ape.config.contracts_folder = contracts_dir
     contracts_dir.mkdir(exist_ok=True, parents=True)
     dependency_contracts_dir.mkdir(exist_ok=True, parents=True)
 
@@ -92,8 +107,11 @@ def with_source_file():
     config_file.unlink(missing_ok=True)
     config_file.write_text(APE_CONFIG_FILE)
 
+    with ape.config.using_project(base_dir) as project:
+        yield project
 
-@pytest.fixture
+
+@pytest.fixture(scope="session")
 def address():
     return CONTRACT_ADDRESS
 
@@ -178,8 +196,14 @@ class MockEtherscanBackend:
         com_testnet_url = get_url_f(testnet=True, tld="com")
 
         return {
-            "ethereum": {"mainnet": url("etherscan"), "goerli": testnet_url("goerli", "etherscan")},
-            "arbitrum": {"mainnet": url("arbiscan"), "goerli": testnet_url("goerli", "arbiscan")},
+            "ethereum": {
+                "mainnet": url("etherscan"),
+                "goerli": testnet_url("goerli", "etherscan"),
+            },
+            "arbitrum": {
+                "mainnet": url("arbiscan"),
+                "goerli": testnet_url("goerli", "arbiscan"),
+            },
             "fantom": {
                 "opera": com_url("ftmscan"),
                 "testnet": com_testnet_url("testnet", "ftmscan"),
@@ -193,12 +217,23 @@ class MockEtherscanBackend:
                 "mumbai": com_testnet_url("testnet", "polygonscan"),
             },
             "avalanche": {"mainnet": url("snowtrace")},
+            "bsc": {
+                "mainnet": com_url("bscscan"),
+                "testnet": com_testnet_url("testnet", "bscscan"),
+            },
         }
 
     def set_network(self, ecosystem: str, network: str):
         self._expected_base_uri = self.expected_uri_map[ecosystem][network.replace("-fork", "")]
 
-    def add_handler(self, method, module, expected_params, return_value=None, side_effect=None):
+    def add_handler(
+        self,
+        method: str,
+        module: str,
+        expected_params: Dict,
+        return_value: Optional[Any] = None,
+        side_effect: Optional[Callable] = None,
+    ):
         if isinstance(return_value, (str, dict)):
             return_value = self.get_mock_response(return_value)
 
@@ -209,7 +244,15 @@ class MockEtherscanBackend:
                     # Allow skipping certain assertions
                     continue
 
-                assert actual_params[key] == val, key
+                # Handler StringIO objects
+                if isinstance(val, StringIO):
+                    assert isinstance(actual_params[key], StringIO)
+                    actual_json = json.loads(actual_params[key].read())
+                    expected_json = json.loads(val.read())
+                    assert actual_json == expected_json
+
+                else:
+                    assert actual_params[key] == val, key
 
             if return_value:
                 return return_value
@@ -273,8 +316,43 @@ class MockEtherscanBackend:
         response = self._mocker.MagicMock(spec=Response)
         response.json.return_value = response_data
         response.text = json.dumps(response_data)
+        response.status_code = 200
 
         for key, val in kwargs.items():
             setattr(response, key, val)
 
         return response
+
+
+@pytest.fixture
+def verification_params(address):
+    ctor_args = "0000000000000005000000000000000000000000000000000000000000000000000000000000000500000000000000000000000000000000000000000000000000000000000000c00000000000000000000000000000000000000000000000000000000000000120000000000000000000000000000000000000000000000000000000000000002a73333362346563316232316330313364393230366536333836653231383935356635616630656533636200000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000002833623465633162323163303133643932303665363338366532313839353566356166306565336362000000000000000000000000000000000000000000000000"  # noqa: E501
+
+    return {
+        "action": "verifysourcecode",
+        "codeformat": "solidity-standard-json-input",
+        "constructorArguements": ctor_args,
+        "contractaddress": address,
+        "contractname": "foo.sol:foo",
+        "evmversion": None,
+        "licenseType": 1,
+        "module": "contract",
+        "optimizationUsed": 1,
+        "runs": 200,
+        "sourceCode": StringIO(json.dumps(STANDARD_INPUT_JSON)),
+    }
+
+
+@pytest.fixture(scope="session")
+def address_to_verify(address, project):
+    contract_type = project.get_contract("foo").contract_type
+    ape.chain.contracts._local_contract_types[address] = contract_type
+    return address
+
+
+@pytest.fixture(scope="session")
+def expected_verification_log(address_to_verify):
+    return (
+        "Contract verification successful!\n"
+        f"https://etherscan.io/address/{address_to_verify}#code"
+    )
